﻿//////////////////////////////////////////////
// GlesShader.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics::gles
{
	class GlesCommandList ;
	class GlesShader ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../../../Shaders/Shader.h"

#include "GlesShaderDataFeeder.h"

/// Class ------------------------------------

namespace nkGraphics::gles
{
	class GlesShader final : public Shader
	{
		private :

			// Attributes
			// Feeder
			GlesShaderDataFeeder _dataFeeder ;

		private :

			// Functions
			virtual ConstantBuffer* allocateConstantBuffer () override ;
			virtual ShaderInstanceMemorySlot* allocateInstanceMemorySlot () override ;

		public :

			// Constructor, destructor
			GlesShader () ;
			~GlesShader () ;

			// Loading
			virtual bool load () override ;
			virtual void unload () override ;

			// Preparations
			void prepareForComputePassUse (GlesCommandList* cBuffer) ;
			void prepareForPostProcessPassUse (GlesCommandList* cBuffer) ;
			void prepareForScenePassUse (GlesCommandList* cBuffer) ;

			bool prepareForRenderable (QueuedRenderableObject* renderable, GlesCommandList* cBuffer) ;

			void noticeComputeShaderActivated (GlesShader* lastUsedShader, GlesCommandList* cBuffer) ;
			void noticeGraphicsShaderActivated (GlesShader* lastUsedShader, GlesCommandList* cBuffer) ;

			void preCommandBufferExecution () ;
			void postComputeCommandBufferExecution () ;
			void postGraphicsCommandBufferExecution () ;

			// Ajouts pour les stratégies
			const std::vector<ConstantBufferResourceDesc>* getConstantBuffersSlots () const ;
			const std::vector<ShaderResourceDesc>* getShaderResourcesSlots () const ;
			const std::vector<SamplerResourceDesc>* getSamplerSlots () const ;
			const std::vector<UavResourceDesc>* getUavBufferSlots () const ;
			const std::vector<ShaderInstanceMemorySlot*>* getInstanceSlots () const ;

			// Surcharge opérateur
			bool operator< (const GlesShader& other) const ;
	} ;
}